; ##############################################################################
; ##############################################################################
; ##############################################################################
;  CPU            SCL															
;1.000.000	=	 27.778															
;2.000.000	=	 55.556															
;4.000.000	=	111.111															
;8.000.000	=	222.222															
;16.000.000	=	444.444															
TWI_INITIALISIERUNG:
.equ TWI_BIT_RATE				=10				; 10	; Bit-Rate							
.equ TWI_PRESCALER				=0				; 0		; Vorteiler: 0=1 1=4 2=16 3=64		
	
	
	; TWI TWBR 				Bit RATE Register	
	ldi temp, TWI_BIT_RATE
	out TWBR, temp
	; TWI TWSR 				STATUS-Register		
	ldi temp, TWI_PRESCALER
	out TWSR, temp
	; TWI TWAR (nur SLAVE) 	Adress-Register		
	clr temp
	out TWAR,temp
	; TWI TWCR 				Control-REGISTER	
	ldi temp, 1<<TWINT|1<<TWEA|0<<TWSTA|0<<TWSTO|0<<TWWC|1<<TWEN|0<<TWIE
	out TWCR,temp

	sbi PORTC,4	; Pull Up on	
	sbi PORTC,5

ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TWI_MT_START:
	clr ERROR
	WDR
	ldi temp, (1<<TWINT)|(1<<TWSTA)|(1<<TWEN)	; START							
	out TWCR, temp	
	rcall TWI_TWINT_wait						; warten, bis TWINT wieder = 1	
	in temp,TWSR								; Check TWSR					
	andi temp, 0xF8
	cpi temp,  0x08								; 0x08 START erfolgreich		
	breq TWI_MT_START_w
	cpi temp,  0x10								; 0x10 (repeated START)			
	brne TWI_MT_START_ERROR						; Error							
;-------------------------------------------------------------------------------
TWI_MT_START_w:
	LDS temp,(adr_TWI_ADR)							; Load SLA + "W"  (W=0) 		
	LSL  temp									; 1 x links
	out TWDR,temp
	ldi temp, (1<<TWINT) | (1<<TWEN)			; senden - Clear TWINT 			
	out TWCR, temp
	rcall TWI_TWINT_wait						; warten, bis TWINT wieder = 1	
	in temp,TWSR								; Check TWSR					
	andi temp, 0xF8
	cpi temp,  0x18								; 0x18							
	brne TWI_MT_START_ERROR						; Error							
ret
;-------------------------------------------------------------------------------
; ERROR																			
TWI_MT_START_ERROR:
	ldi temp,(ERROR_TWI_MT_START)
	mov ERROR, temp
	LED_ROT_ON
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; inp: TEMP																		
TWI_MT_TX_TEMP:
	WDR
	tst ERROR
	breq TWI_MT_TX_RUN
	ret
;-------------------------------------------------------------------------------
TWI_MT_TX_RUN:
	out TWDR, temp 							; 	Load DATA						
	ldi temp, (1<<TWINT) | (1<<TWEN)		; senden - Clear TWINT 				
	out TWCR, temp
	rcall TWI_TWINT_wait					; warten, bis TWINT wieder = 1		
	in temp,TWSR							; Check TWSR						
	andi temp, 0xF8
	cpi temp, 0x28							; 0x28								
	brne TWI_MT_ERROR
	clr temp
	ret
;-------------------------------------------------------------------------------
; ERROR																			
TWI_MT_ERROR:
	ldi temp,(ERROR_TWI_MT_DATA)
	mov ERROR, temp
	LED_ROT_ON
	clr temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TWI_MASTER_RX_START:
	WDR
	clr ERROR
	STS	(adr_TWI_BEREICH+0),NULL			; SRAM lschen						
	ldi temp, (1<<TWINT | 1<<TWEA | 1<<TWSTA | 1<<TWEN ); START 				
	out TWCR, temp	
	rcall TWI_TWINT_wait					; warten, bis TWINT wieder = 1		
	in temp,TWSR							; Check TWSR 						
	andi temp, 0xF8
	cpi temp,  0x08 						; 0x08 START erfolgreich			
	breq TWI_MASTER_RX_w
	cpi temp,  0x10							; 0x10 (repeated START)				
	brne TWI_MASTER_RX_ERROR
;-------------------------------------------------------------------------------
TWI_MASTER_RX_w:
	LDS temp,(adr_TWI_ADR)					; Load SLA +"R" (R=1)				
	lsl  temp								; 1 x links							
	ori temp, 0b00000001 
	out TWDR, temp
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )	; senden - Clear TWINT 	
	out TWCR, temp
	rcall TWI_TWINT_wait					; warten, bis TWINT wieder = 1		
	in temp,TWSR							; Check TWSR						
	andi temp, 0xF8
	cpi temp, 0x40 							;SLA+R gesendet; ACK empfangen		
	brne TWI_MASTER_RX_ERROR
	ret
;-------------------------------------------------------------------------------
; ERROR																			
TWI_MASTER_RX_ERROR:
	ldi temp,(ERROR_TWI_MR_START)
	mov ERROR, temp
	LED_ROT_ON
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Achtung, diese Routine verwenden, wenn 										
; mehrere! BYTES empfangen werden sollen										
;																				
; OUTPUT: temp1																	
;																				
TWI_MASTER_RX_DATA_TEMP1:
	tst ERROR
	breq TWI_MASTER_RX_DATA_RUN
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RX_DATA_RUN:
	ldi temp, (1<<TWINT | 1<<TWEA  | 1<<TWEN )	; senden "ACK" (TWEA=1)			
	out TWCR, temp
	rcall TWI_TWINT_wait						; TWINT abfragen				
	in temp,TWSR								; Check TWSR					
	andi temp, 0xF8
	cpi temp, 0x50								; Data Byte empfangen- ACK zurckgesendet
	brne TWI_MASTER_RX_DATA_ERROR
	in temp1, TWDR								; DATEN einlesen				
ret
;-------------------------------------------------------------------------------
TWI_MASTER_RX_DATA_ERROR:
	ldi temp,(ERROR_TWI_MR_DATA)
	mov ERROR, temp	
	LED_ROT_ON
	clr temp1
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Achtung, diese Routine verwenden, wenn 										
; nur ein! Bytes empfangen werden soll, oder wenn								
; das letzte! Byte empfangen werden soll										
;																				
; OUTPUT: temp1																	
;																				
TWI_MASTER_RX_LAST_BYTES:
	tst ERROR
	breq TWI_MASTER_RX_DATA_LAST_BYTES_RUN
	ret
;-------------------------------------------------------------------------------
TWI_MASTER_RX_DATA_LAST_BYTES_RUN:

	ldi temp, (1<<TWINT | 0<<TWEA  | 1<<TWEN )	; senden "NOT ACK" (TWEA=0)		
	out TWCR, temp

	rcall TWI_TWINT_wait						; TWINT abfragen				

	in temp,TWSR								; Check TWSR					
	andi temp, 0xF8
	cpi  temp, 0x58								; letztes Data Byte empfangen-NOT ACK zurckgesendet
	brne TWI_MASTER_RX_DATA_LAST_BYTES_ERROR	; ERROR							
	in temp1, TWDR								; BYTES einlesen				
ret
;-------------------------------------------------------------------------------
; ERROR																			
TWI_MASTER_RX_DATA_LAST_BYTES_ERROR:
	ldi temp,(ERROR_TWI_MR_DATAEND)
	mov ERROR, temp	
	LED_ROT_ON
	clr temp1
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; STOP- Routine																   	
TWI_STOP:
	ldi temp, (1<<TWINT)|(1<<TWEN)|(1<<TWSTO)|0<<TWIE
	out TWCR, temp	
	;rcall WAIT_1ms
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; warten, bis das TWINT Flag gesetzt ist									   	
TWI_TWINT_WAIT:
	LED_GELB_ON
TWI_TWINT_WAIT_s:
	in temp, TWCR
	sbrs temp, TWINT	; nchsten Befehl berspringen, wenn TWINT FLAG  gesetzt ist
	rjmp TWI_TWINT_WAIT_s
	LED_GELB_OFF
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
