;												
; Schrittmotor-Tester ATmega8					
; Bernhard.Erfurt@gmx.de						
; 01/2021										
;												
; LCD 2x16										
; Motor NEMA17-05GM 1036 Stepps 5,18:1			
; 												
;												
;												
; LED ROT blinkt: Programm Check Error			
; LED ROT+GELB+GRN blinkt: Interrupt Error		
;												

; ATmega8 										
.include "m8def.inc"
.include "MACRO.asm"
; KONSTANTEN									
.equ VERSION						=1
.equ TAKT							=16000000	; 16 MHz	
; MOTOR											
.equ MOTOR_STEPS					=200
.equ MIKROSTEPS_PRO_STEP			=360
.equ GETRIEBE						=518		; 518=5.18:1	
.equ SEKUNDEN_PRO_TAG				=864000		; 24*60*60*10	
; Error-Codes									
.equ ERROR_TWI_MT_START				=0x20
.equ ERROR_TWI_MT_SLA				=0x21
.equ ERROR_TWI_MT_DATA				=0x22
.equ ERROR_TWI_MR_START				=0x23
.equ ERROR_TWI_MR_SLA				=0x24
.equ ERROR_TWI_MR_DATA				=0x25
.equ ERROR_TWI_MR_DATAEND			=0x26
; PWM											
.equ PWM_A_L						=OCR1AL		; MOTOR-A WICKLUNG 	
.equ PWM_A_H						=OCR1AH
.equ PWM_B_L						=OCR1BL		; MOTOR-B WICKLUNG	
.equ PWM_B_H						=OCR1BH
; TASTE "A"										
.equ TASTE_A_PIN_NR					= 0
.equ TASTE_A_PIN					= PINC
.equ TASTE_A_DDR					= DDRC
.equ TASTE_A_PORT					= PORTC
; TASTE "B"										
.equ TASTE_B_PIN_NR					= 1
.equ TASTE_B_PIN					= PINC
.equ TASTE_B_DDR					= DDRC
.equ TASTE_B_PORT					= PORTC

; MOTOR-A0 WICKLUNG 							
.equ A0_PIN_NR						= 0
.equ A0_DDR							= DDRD
.equ A0_PORT						= PORTD
; MOTOR-A1 WICKLUNG 							
.equ A1_PIN_NR						= 1
.equ A1_DDR							= DDRB
.equ A1_PORT						= PORTB

; MOTOR-B0 WICKLUNG 							
.equ B0_PIN_NR						= 1
.equ B0_DDR							= DDRD
.equ B0_PORT						= PORTD
; MOTOR-B1 WICKLUNG 							
.equ B1_PIN_NR						= 2
.equ B1_DDR							= DDRB
.equ B1_PORT						= PORTB
; LED-GRUEN										
.equ LED_GRUEN_PIN_NR				= 5
.equ LED_GRUEN_DDR					= DDRD
.equ LED_GRUEN_PORT					= PORTD
; LED-GELB										
.equ LED_GELB_PIN_NR				= 6
.equ LED_GELB_DDR					= DDRD
.equ LED_GELB_PORT					= PORTD
; LED-ROT										
.equ LED_ROT_PIN_NR					= 7
.equ LED_ROT_DDR					= DDRD
.equ LED_ROT_PORT					= PORTD
; ##############################################################################
; ##############################################################################
; ##############################################################################
; SRAM-BELEGUNG ATmega8															
.equ adr_DATEN						=SRAM_START		; 0x0060 START Adresse 		
.equ adr_VERSION					=SRAM_START+0

.equ adr_GERTRIEBE_L				=SRAM_START+2	; Getriebebersetzung		
.equ adr_GERTRIEBE_H				=SRAM_START+3


.equ adr_1ms						=SRAM_START+20
.equ adr_10ms						=SRAM_START+21
.equ adr_100ms						=SRAM_START+22
.equ adr_ss							=SRAM_START+23	; Sekunden					
.equ adr_mm							=SRAM_START+24	; Minuten					
.equ adr_hh							=SRAM_START+25	; Stunden

.equ adr_TASTE						=SRAM_START+26
.equ adr_MENUE						=SRAM_START+27
.equ adr_MENUE_TIMER				=SRAM_START+28
.equ adr_ADC_BETRIEBSSPANNUNG		=SRAM_START+29

.equ adr_KONSTANTE_L				=SRAM_START+80
.equ adr_KONSTANTE_H				=SRAM_START+81

.equ adr_POSITION_SOLL_0			=SRAM_START+90
.equ adr_POSITION_SOLL_1			=SRAM_START+91
.equ adr_POSITION_SOLL_2			=SRAM_START+92

.equ adr_POSITION_IST_0				=SRAM_START+93
.equ adr_POSITION_IST_1				=SRAM_START+94
.equ adr_POSITION_IST_2				=SRAM_START+95

.equ adr_DREHFELD_L					=SRAM_START+96	; 0...720(0...360) 
.equ adr_DREHFELD_H					=SRAM_START+97

.equ adr_ASCII_FONT					= SRAM_START+100
.equ adr_ASCII_BEREICH_OZ			= SRAM_START+101 ; obere  Zeile				
.equ adr_ASCII_BEREICH_UZ			= SRAM_START+120 ; untere Zeile	

.equ adr_TWI_ADR					= SRAM_START+200
.equ adr_TWI_BEREICH				= SRAM_START+201	; BEREICH fr Master TWI/I2C	

.equ adr_SRAM_CHECK					=SRAM_START+943	; SRAM CHECK muss immer "S" sein	
; ##############################################################################
; ##############################################################################
; ##############################################################################
; REGISTER										

.def ZEITZAHL_0		= R5
.def ZEITZAHL_1		= R6
.def ZEITZAHL_2		= R7
.def ZEITZAHL_3		= R8

.def TIMERWERT0		= R9
.def ERROR			= R10
.def OLED_X 		= R11
.def OLED_Y 		= R12
.def NULL 			= R13
.def EINS 			= R14
.def VOLL 			= R15
.def temp 			= R16
.def temp1			= R17
.def temp2			= R18
.def temp3			= R19
.def temp4			= R20
.def temp5			= R21
.def temp6			= R22
.def temp7			= R23
.def temp8			= R24
.def temp9			= R25


.cseg					;Beginn eines Code-Segmentes
.org 0					;Startadresse=0

rjmp RESET      		; Reset Handler                           				
rjmp INTERRUPT_ERROR	;rjmp EXT_INT0   ; IRQ0 Handler                      	
rjmp INTERRUPT_ERROR	;rjmp EXT_INT1   ; IRQ1 Handler                      	
rjmp INTERRUPT_ERROR	;rjmp TIM2_COMP  ; Timer2 Compare Handler            	
rjmp TIM2_OVF   	; Timer2 Overflow Handler           						
rjmp INTERRUPT_ERROR	;rjmp TIM1_CAPT  ; Timer1 Capture Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPA ; Timer1 CompareA Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPB ; Timer1 CompareB Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_OVF   	  	; Timer1 Overflow Handler       
rjmp TIM0_OVF   	; Timer0 Overflow Handler           						
rjmp INTERRUPT_ERROR	;rjmp SPI_STC    ; SPI Transfer Complete Handler     	
rjmp INTERRUPT_ERROR	;rjmp ZEICHEN_EINLESEN 	; USART RX Complete Handler     
rjmp INTERRUPT_ERROR	;rjmp USART_UDRE ; UDR Empty Handler                 	
rjmp INTERRUPT_ERROR	;rjmp USART_TXC  ; USART TX Complete Handler         	
rjmp INTERRUPT_ERROR	;rjmp ADC        ; ADC Conversion Complete Handler   	
rjmp INTERRUPT_ERROR	;rjmp EE_RDY     ; EEPROM Ready Handler              	
rjmp INTERRUPT_ERROR	;rjmp ANA_COMP   ; Analog Comparator Handler         	
rjmp INTERRUPT_ERROR	;rjmp TWSI       ; Two-wire Serial Interface Handler 	
rjmp INTERRUPT_ERROR	;rjmp SPM_RDY    ; Store Program Memory Ready Handler	
;-------------------------------------------------------------------------------
; Unkorrekten Interrupt Aufruf 													
INTERRUPT_ERROR:
	LED_GRUEN_ON
	LED_GELB_ON
	LED_ROT_ON
	rcall WAIT_100ms
	LED_GRUEN_OFF
	LED_GELB_OFF
	LED_ROT_OFF
	rjmp INTERRUPT_ERROR
;-------------------------------------------------------------------------------
RESET:                        
	ldi temp, LOW(RAMEND)					; STACK initialisieren								
    out SPL, temp
    ldi temp, HIGH(RAMEND)
    out SPH, temp

	rcall WDR_INITIALISIERUNG				; WDR_INITIALISIERUNG								
	rcall SRAM_CLEAR						; SRAM lschen										
	rcall PORT_CLEAR						; PORT_CLEAR										
	rcall REGISTER_CLEAR					; REGISTER-CLEAR									
	rcall STARTWERTE						; Startwerte										
	rcall PORT_INITIALISIERUNG				; LED_INITIALISIERUNG								
	rcall TIMER0_PHASENUNKEHR_INITIALISIERUNG
	rcall TIMER1_PWM_INITIALISIERUNG
	rcall TIMER2_1ms_INITIALISIERUNG
	rcall TIMER_AKTIVIEREN					; TIMER_AKTIVIEREN									
	rcall ADC_INITIALISIERUNG_VBG
	rcall TWI_INITIALISIERUNG
	rcall TWI_OLED_INITIALISIERUNG
	rcall PROGRAMM_CHECK					; CHECK												
	sei										; Interrupts freigeben								
;-------------------------------------------------------------------------------
;	LED_GRUEN_ON
	rcall OLED_VOLL
	rcall WAIT_100ms
	rcall OLED_CLEAR
	rcall ANZEIGE_BEGRUESSUNG
	rcall WAIT_1s
	rcall OLED_CLEAR
;	rcall EEPROM_LOAD
	rcall MOTOR_INITIALISIERUNG
	rcall MOTOR_SANFTANLAUF
	rcall ANZEIGE_SYSTEM_MOTOR
	rcall WAIT_1s
	rcall OLED_CLEAR
	rcall UHR_NULL
;-------------------------------------------------------------------------------
LOOP:
	WDR
	rcall BERECHNUNG_POSITION_SOLL
	rcall VERGLEICH_MOTOR_SOLL_IST
	LDS temp,(adr_hh)
	tst temp
	breq LOOP
	rcall PROGRAMM_STOP
rjmp LOOP
; ##############################################################################
; ##############################################################################
; ##############################################################################
BERECHNUNG_POSITION_SOLL:
	CLI 					; Global Interrupt Disable	
	mov temp1,ZEITZAHL_1	;laden und durch 256		
	mov temp2,ZEITZAHL_2
	mov temp3,ZEITZAHL_3
	SEI						; Interrupts freigeben		
	LSR temp3
	ROR temp2
	ROR temp1
;-------------------------------------------------------------------------------
	STS(adr_POSITION_SOLL_0),temp1
	STS(adr_POSITION_SOLL_1),temp2
	STS(adr_POSITION_SOLL_2),temp3
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
VERGLEICH_MOTOR_SOLL_IST:
	LDS temp1,(adr_POSITION_SOLL_0)
	LDS temp2,(adr_POSITION_IST_0)
	cp temp1,temp2
	brne VERGLEICH_MOTOR_SOLL_IST_s
	ret
;-------------------------------------------------------------------------------
VERGLEICH_MOTOR_SOLL_IST_s:
	rcall MOTOR_MIKROSTEP
	LDS temp1,(adr_POSITION_SOLL_0)
	LDS temp2,(adr_POSITION_IST_0)
	cp temp1,temp2
	brne VERGLEICH_MOTOR_SOLL_IST_s
	rjmp ANZEIGE_HAUPT
; ##############################################################################
; ##############################################################################
; ##############################################################################
.include "ADC.asm"
;.include "EEPROM.asm"
;.include "MENUE.asm"
.include "TIMER.asm"
.include "MOTOR.asm"
.include "SONSTIGES.asm"
.include "WAIT.asm"
.include "TWI.asm"
.include "TWI_OLED.asm"
.include "ASCII.asm"
.include "ANZEIGE.asm"
.include "ZAHLEN.asm"
;-------------------------------------------------------------------------------
TEXT_BEGRUESSUNG_1:
.db "ASTRO",0
TEXT_BEGRUESSUNG_2:
.db "ATmega328p ",0
TEXT_BEGRUESSUNG_3:
.db "128x64 Pixel ",0
TEXT_BEGRUESSUNG_4:
.db "02/2021 V2 ",0




